unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  Windows, Classes, SysUtils, FileUtil, DateTimePicker, Forms, Controls,
  Graphics, Dialogs, EditBtn, StdCtrls, ComCtrls, CheckLst, Grids, ExtCtrls,
  DJTSatLib_TLB, math;

type

  { TForm1 }

  TForm1 = class(TForm)
    Button1: TButton;
    ButtonTLEUpdate: TButton;
    DateTimePickerDate: TDateTimePicker;
    DateTimePickerTime: TDateTimePicker;
    EditLocator: TEdit;
    EditLongitude: TEdit;
    EditLatitude: TEdit;
    EditAltitude: TEdit;
    GroupBox1: TGroupBox;
    GroupBoxTLEUpdate: TGroupBox;
    LabelLastUpdateText: TLabel;
    LabelLastUpdateValue: TLabel;
    LabelLongitude: TLabel;
    LabelLatitude: TLabel;
    LabelLongitude1: TLabel;
    GroupBoxLocation: TGroupBox;
    LabelLocator: TLabel;
    ListSatNames: TCheckListBox;
    LoadTLEFile: TButton;
    OpenTLEFile: TOpenDialog;
    PageControl: TPageControl;
    StringGrid1: TStringGrid;
    TabSatList: TTabSheet;
    TabS: TTabSheet;
    TabSetup: TTabSheet;
    TimerRealTime: TTimer;
    procedure FormShow(Sender: TObject);
    procedure ListSatNamesClickCheck(Sender: TObject);
    procedure LoadTLEFileClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure PageControlChange(Sender: TObject);
    procedure TimerRealTimeTimer(Sender: TObject);
  private
    { Private declarations }
    ISat: ISatellites;
    procedure date_time_changed;
  public

  end;

var
  Form1: TForm1;
  slSat: TStringList;
  firststart:boolean;

implementation

uses
  ComObj;

{$R *.lfm}

{ TForm1 }

function NowUTC: TDateTime;
var
  system_datetime: TSystemTime;
begin
  GetSystemTime (system_datetime);
  Result := SystemTimeToDateTime (system_datetime);
end;

procedure TForm1.date_time_changed;
const
  Rad2Deg = 180 / pi;
var
  i:integer;
  utc: TDateTime;
  jt : double;
  sat: WideString;
  is_visible: WordBool;
  southbound: WordBool;
  az, el, lon, lat, phase: double;
  st: string;
  range, range_rate, altitude: double;
  rev_number: integer;
  orbit_phase: double;
  ma: double;
  ma_number: integer;
  ma_phase: double;
  aos, los: TDateTime;
  x, y, z, xdot, ydot, zdot: double;

  tname,tval:string;
  slSatCount:integer;
  visibility:string;
  direction:string;
  rlon:string;
  rlat:string;
begin
  if not Assigned (ISat) then exit;
  utc         := trunc(DateTimePickerDate.DateTime) + frac(DateTimePickerTime.DateTime);
  jt          := ISat.DateTimeToJT(utc);

  slSatCount := slSat.Count;
  StringGrid1.RowCount := StringGrid1.FixedRows + slSatCount;
//  if slSatCount > 0 then
//  begin
    for i:=0 to slSatCount - 1 do
    begin
      // read name from stringlist
      slSat.GetNameValue(i,tname,tval);
      sat         := tval;
      // get calculated data from dll
      is_visible  := ISat.IsVisible(sat,jt,0.5,southbound,az,el,lon,lat,range,range_rate,altitude,phase);
      rev_number  := ISat.RevNumber;
      orbit_phase := ISat.OrbitPhase;
      // set rows
      //StringGrid1.RowCount := StringGrid1.FixedRows + i + 1;

      if is_visible then visibility:='ja' else visibility:='nein';
      if southbound then direction:='S' else direction:='N';

      if radtodeg(lon)>180 then
      begin lon:=360-radtodeg(lon); rlon := 'W'; end else
      begin lon:=radtodeg(lon); rlon := 'O'; end;

      if radtodeg(lat)<0 then
      begin lat:=lat * -1; rlat := 'S'; end
      else rlat := 'N';

      // fill the grid
      StringGrid1.Cells[0,i+1]  := sat;
      StringGrid1.Cells[1,i+1]  := visibility;
      StringGrid1.Cells[2,i+1]  := direction;
      StringGrid1.Cells[3,i+1]  := Format('%.2f°',[radtodeg(az)]);
      StringGrid1.Cells[4,i+1]  := Format('%.2f°',[radtodeg(el)]);
      StringGrid1.Cells[5,i+1]  := Format('%.4f° '+rlon,[lon]);
      StringGrid1.Cells[6,i+1]  := Format('%.4f° '+rlat,[radtodeg(lat)]);
      StringGrid1.Cells[7,i+1]  := Format('%.3f',[range]);
      StringGrid1.Cells[8,i+1]  := Format('%.3f',[range_rate]);
      StringGrid1.Cells[9,i+1]  := Format('%.3f',[altitude]);
      StringGrid1.Cells[10,i+1] := Format('%.1f°',[radtodeg(orbit_phase)]);
    end;
//  end;
end;


procedure TForm1.FormCreate(Sender: TObject);
begin
  DefaultFormatSettings.DecimalSeparator := '.';
  firststart:=true;
  //
  Width:=247;
  // set stringgrid
  StringGrid1.FixedRows:=1;
  StringGrid1.RowCount:=1;
  // create stringlist for selected satnames in checklistbox
  slSat  := TStringList.Create;
  // set tab to setup
  PageControl.ActivePageIndex:=0;


  try
    // Try and connect to the Active-X DLL
    ISat := CreateComObject (CLASS_Satellites) as Satellites;
  except
  end;

  if Assigned (ISat)
  then
    begin
      ISat.ObsLon:=13.794565;
      ISat.ObsLat:=50.912703;
      ISat.ObsHeight:=387.0;
    end
  else
    begin
    ShowMessage ('Error connecting to in-process COM server DJTSatLib.dll' + #13 + #13 +
                 'Try using the command "REGSVR32 DJTSatLib.dll" ' +
                 'from the command prompt in the folder containing ' +
                 'the file DJTSatLib.dll.' + #13 + #13 +
                 'The REGSVR32 command may be found in your Windows System folder.');
    end;
end;

procedure TForm1.LoadTLEFileClick(Sender: TObject);
var
  TLEFile: string;
begin
  OpenTLEFile.FileName:=ExtractFilePath(Application.ExeName)+'keps\';
  OpenTLEFile.Filter:='*.tle,*.txt';
  if OpenTLEFile.Execute then
  begin
    ListSatNames.Clear;
    TLEFile := OpenTLEFile.Filename;
    ISat.KeplerPath:=TLEFile;
    ListSatNames.Items.CommaText:=ISat.SatNames;
  end;
end;

procedure TForm1.ListSatNamesClickCheck(Sender: TObject);
var
  i,d: integer;
begin
  slSat.Clear;
  d := 0;
  for i := 0 to ListSatNames.Items.Count - 1 do
    if ListSatNames.State[i] = cbChecked then
    begin
      slSat.Add(ListSatNames.Items[i]);
      inc(d);
    end;

  //
  //StringGrid1.RowCount := StringGrid1.FixedRows + d;

  date_time_changed;
end;

procedure TForm1.FormShow(Sender: TObject);
begin
  DateTimePickerDate.DateTime:= NowUTC;
  DateTimePickerTime.DateTime:= NowUTC;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  // free
  slSat.Free;

  // clear dll
  if Assigned (ISat) then
    begin
    ISat.KeplerPath := '';
    ISat := nil;
    end;
end;

procedure TForm1.PageControlChange(Sender: TObject);
begin
  if not firststart then
  begin
    PageControl.ActivePageIndex:=0;
    ShowMessage('Please first fill in all needed information!!');
  end
  else
  begin
    case PageControl.ActivePageIndex of
      0: begin
        Width:=247;
      end;
      1: begin
        Width:=1003;
      end;
    end;
  end;
end;

procedure TForm1.TimerRealTimeTimer(Sender: TObject);
begin

  DateTimePickerDate.DateTime:= NowUTC;
  DateTimePickerTime.DateTime:= NowUTC;
  if StringGrid1.RowCount > 1 then
  date_time_changed;
end;


end.

